/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

import com.tplan.robot.ApplicationSupport;
import com.tplan.robot.gui.MainFrame;
import com.tplan.robot.preferences.UserConfiguration;
import com.tplan.robot.remoteclient.adb.LibLoader;
import com.tplan.robot.remoteclient.adb.TimeoutException;
import com.tplan.robot.util.Utils;
import java.awt.Cursor;
import java.awt.EventQueue;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Arrays;
import java.util.List;
import javax.swing.Icon;
import javax.swing.SwingUtilities;

/**
 * ADB library loader helper. It ensures that the libraries from the Android SDK are
 * loaded when this plugin is being initialized.
 */
public class AdbLoaderHelper implements WindowListener, Runnable {

    private Throwable t;
    private boolean firstRun = true;

    AdbLoaderHelper(Throwable t) {
        this.t = t;
    }

    static void testLibs() throws Throwable {
        Class.forName("com.android.ddmlib.AndroidDebugBridge");
    }

    static void loadLibs() throws Throwable {
        // Update in 3.0 - The previously dummy connect used to load the libraries 
        // caused a deadlock while another connection was in progress (23 March 2015).
        // That's why we replaced it with a direct call of the AdbManager class.
        LibLoader.loadLibs();
    }

    public void run() {
        if (firstRun) {
            //System.out.println("First run");
            try {
                String args[] = ApplicationSupport.getInputArguments();
                boolean cliMode = false;
                if (args != null) {
                    List<String> largs = Arrays.asList(args);
                    if (largs.contains("-n") || largs.contains("--nodisplay")) {
                        cliMode = true;
                    }
                }
                if (!cliMode && !EventQueue.isDispatchThread()) {
                    //System.out.println("  ... waiting for the main frame to pop up");
                    // Wait for the MainFrame to get populated not longer than 10 seconds.
                    long endTime = System.currentTimeMillis() + 10000;
                    MainFrame mf;
                    while ((mf = MainFrame.getInstance()) == null && System.currentTimeMillis() < endTime) {
                        Utils.sleep(50);
                    }
                    if (mf != null) {
                        if (!mf.isShowing()) {
                            //System.out.println("  ... not showing -> adding a window listener");
                            mf.addWindowListener(this);
                            return;
                        }
                    }
                }
            } finally {
                firstRun = false;
            }
            // Reexecute this runnable
            EventQueue.invokeLater(this);
        } else {
            MainFrame frame = MainFrame.getInstance();
            if (frame == null) {
                System.err.println("ERROR: The script uses the ADB Extension plugin but the "
                        + "path to the Android SDK is not configured or it is invalid. Please read "
                        + "the plugin documentation at " + Constants.PLUGIN_DOC_URL + " for the instructions "
                        + "on how to configure the path.");
            } else {
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException ex) {
                }
                String options[] = new String[]{
                    "Configure the path",
                    "Ignore"
                };
                Icon icons[] = new Icon[]{
                    ApplicationSupport.getImageIcon("preferences16.gif"),
                    ApplicationSupport.getImageIcon("exit16.png")
                };

                boolean exit = false;
                do {
                    String path = UserConfiguration.getInstance().getString("adb.path");
                    String msg;
                    if (path == null || path.trim().isEmpty()) {
                        msg = "is not configured";
                    } else {
                        msg = "of <code>" + path + "</code> is invalid";
                    }
                    int option = Utils.showMessageDialog(frame, "ADB Extension Error",
                            "<html><body>This Robot instance uses the <b>ADB Extension</b> plugin but the "
                            + "path to the Android SDK " + msg + ". Please read "
                            + "the <a href=\"" + Constants.PLUGIN_DOC_URL + "\">plugin documentation</a> for the instructions "
                            + "on how to configure the path. Alternatively select the <b><i>" + options[0]
                            + "</i></b> button, provide a valid path and then recompile the scripts to "
                            + "resolve the syntax errors.</body></html>",
                            t, options, icons, options[0]);
                    if (option == 0) {
                        // Show the "Android Over ADB" preference window
                        frame.showOptionsDialog(ApplicationSupport.getString("adbClient.pluginName"), null);
                        final Cursor origCursor = frame.getCursor();
                        try {
                            frame.setCursor(new Cursor(Cursor.WAIT_CURSOR));
                            // Load the libs
                            loadLibs();
                            testLibs();
                            exit = true;
                        } catch (TimeoutException ex) {
                            // Ignore the timeout exception
                            exit = true;
                        } catch (Throwable ex) {
                            t = ex;
                        } finally {
                            frame.setCursor(origCursor);
                        }
                    } else {
                        exit = true;
                    }
                } while (!exit);
            }
        }
    }

    public void windowOpened(WindowEvent e) {
        e.getWindow().removeWindowListener(this);
        SwingUtilities.invokeLater(this);
    }

    public void windowClosing(WindowEvent e) {
    }

    public void windowClosed(WindowEvent e) {
    }

    public void windowIconified(WindowEvent e) {
    }

    public void windowDeiconified(WindowEvent e) {
        windowOpened(e);
    }

    public void windowActivated(WindowEvent e) {
        windowOpened(e);
    }

    public void windowDeactivated(WindowEvent e) {
    }
}
